package com.envers.test;

import java.util.List;

import junit.framework.Assert;

import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.classic.Session;
import org.hibernate.envers.AuditReader;
import org.hibernate.envers.AuditReaderFactory;
import org.hibernate.envers.query.AuditEntity;
import org.junit.Before;
import org.junit.Test;

import com.envers.entity.Campaign;

public class DuplicatedRevisionsTest {

	private SessionFactory sessionFactory;

	@Before
	public void setUp() {

		sessionFactory = new AnnotationConfiguration().configure()
				.buildSessionFactory();
	}

	@Test
	public void testDuplicatedRevisionCreation() {

		Session session1 = sessionFactory.openSession();

		Transaction transaction = session1.beginTransaction();

		transaction.begin();

		// Save identical entities
		Campaign firstCampaign = new Campaign();
		firstCampaign.setDescription("the description");

		Long id = (Long) session1.save(firstCampaign);

		transaction.commit();

		Session session2 = sessionFactory.openSession();

		transaction = session2.beginTransaction();

		Campaign secondCampaign = new Campaign();
		secondCampaign.setId(Long.valueOf(id));
		secondCampaign.setDescription("the description");

		// Save the second campaign which is exactly the same as the first one
		// no new records are generated in the operational table
		session2.update(secondCampaign);

		transaction.commit();

		AuditReader reader = AuditReaderFactory.get(session2);

		// Retrieve the recently created revisions
		@SuppressWarnings("unchecked")
		List<Campaign> campaignsDelta = reader.createQuery()
				.forRevisionsOfEntity(Campaign.class, true, true)
				.add(AuditEntity.id().eq(Long.valueOf(id))).getResultList();

		// The number of revisions should be 1 as no entities have been modified
		Assert.assertEquals(
				"Assert whether there is more than one revision for the unmodified campaign",
				1, campaignsDelta.size());
	}
}
